@@ -5,7 +5,7 @@
 /*                                                                           */
 /* Copyright (C) 2011-2013 Discrete Optimization, TU Darmstadt,              */
 /*                         EDOM, FAU Erlangen-Nürnberg                       */
-/*               2014-2022 Discrete Optimization, TU Darmstadt               */
+/*               2014-2024 Discrete Optimization, TU Darmstadt               */
 /*                                                                           */
 /*                                                                           */
 /* Licensed under the Apache License, Version 2.0 (the "License");           */
@@ -22,7 +22,7 @@
 /*                                                                           */
 /*                                                                           */
 /* Based on SCIP - Solving Constraint Integer Programs                       */
-/* Copyright (C) 2002-2022 Zuse Institute Berlin                             */
+/* Copyright (C) 2002-2024 Zuse Institute Berlin                             */
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -47,7 +47,7 @@
 #include <assert.h>
 
 #include "lapack_interface.h"
-#include "config.h"                          /* for F77_FUNC */
+#include "configf77.h"                          /* for F77_FUNC */
 
 #include "scip/def.h"
 #include "scip/pub_message.h"                /* for debug and error message */
@@ -57,10 +57,18 @@
 /* turn off lint warnings for whole file: */
 /*lint --e{788,818}*/
 
-
-/* we use 64 bit integers as the base type */
+/* use int type from Openblas if available */
+#ifdef LAPACKLONG
 typedef long long int LAPACKINTTYPE;
-
+#else
+#ifdef OPENBLAS
+#include <cblas.h>
+typedef blasint LAPACKINTTYPE;
+#else
+/* otherwise we use int as a default */
+typedef int LAPACKINTTYPE;
+#endif
+#endif
 
 /** Checks if a BMSallocMemory-call was successfull, otherwise returns SCIP_NOMEMORY */
 #define BMS_CALL(x)   do                                                                                      \
