@@ -5,7 +5,7 @@
 /*                                                                           */
 /* Copyright (C) 2011-2013 Discrete Optimization, TU Darmstadt,              */
 /*                         EDOM, FAU Erlangen-Nürnberg                       */
-/*               2014-2022 Discrete Optimization, TU Darmstadt               */
+/*               2014-2024 Discrete Optimization, TU Darmstadt               */
 /*                                                                           */
 /*                                                                           */
 /* Licensed under the Apache License, Version 2.0 (the "License");           */
@@ -22,7 +22,7 @@
 /*                                                                           */
 /*                                                                           */
 /* Based on SCIP - Solving Constraint Integer Programs                       */
-/* Copyright (C) 2002-2022 Zuse Institute Berlin                             */
+/* Copyright (C) 2002-2024 Zuse Institute Berlin                             */
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -58,37 +58,38 @@
 SCIP_RETCODE SCIPsdpSolcheckerCheck(
    BMS_BUFMEM*           bufmem,             /**< buffer memory */
    int                   nvars,              /**< number of variables */
-   SCIP_Real*            lb,                 /**< lower bounds of variables */
-   SCIP_Real*            ub,                 /**< upper bounds of variables */
+   const SCIP_Real*      lb,                 /**< lower bounds of variables */
+   const SCIP_Real*      ub,                 /**< upper bounds of variables */
    int                   nsdpblocks,         /**< number of SDP-blocks */
-   int*                  sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
-   int*                  sdpnblockvars,      /**< number of variables that exist in each block */
+   const int*            sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
+   const int*            sdpnblockvars,      /**< number of variables that exist in each block */
    int                   sdpconstnnonz,      /**< number of nonzero elements in the constant matrices of the SDP-blocks AFTER FIXINGS */
-   int*                  sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
+   const int*            sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
                                               *   number of entries  of sdpconst row/col/val [i] AFTER FIXINGS */
-   int**                 sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
-   int**                 sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
-   SCIP_Real**           sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
+   int* const*           sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
+   int* const*           sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
+   SCIP_Real* const*     sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
    int                   sdpnnonz,           /**< number of nonzero elements in the SDP-constraint-matrix */
-   int**                 sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
+   int* const*           sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
                                               *   the number of entries of sdp row/col/val [i][j] */
-   int**                 sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
+   int* const*           sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
                                               *   in the i-th block */
-   int***                sdprow,             /**< pointer to the row-indices for each block and variable */
-   int***                sdpcol,             /**< pointer to the column-indices for each block and variable */
-   SCIP_Real***          sdpval,             /**< values of SDP-constrain tmmatrix entries (may be NULL if sdpnnonz = 0) */
-   int**                 indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
+   int** const*          sdprow,             /**< pointer to the row-indices for each block and variable */
+   int** const*          sdpcol,             /**< pointer to the column-indices for each block and variable */
+   SCIP_Real** const*    sdpval,             /**< values of SDP-constraint mmatrix entries (may be NULL if sdpnnonz = 0) */
+   int* const*           indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
                                               *   be removed, otherwise it gives the number of indices removed before this */
-   int*                  nremovedinds,       /**< the number of rows/cols to be fixed for each block */
-   int*                  blockindchanges,    /**< block indizes will be modified by these, see indchanges */
+   const int*            nremovedinds,       /**< the number of rows/cols to be fixed for each block */
+   const int*            blockindchanges,    /**< block indizes will be modified by these, see indchanges */
    int                   nlpcons,            /**< number of active (at least two nonzeros) LP-constraints */
-   SCIP_Real*            lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
-   SCIP_Real*            lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const int*            lpindchanges,       /**< array for the number of LP-constraints removed before the current one (-1 if removed itself) */
+   const SCIP_Real*      lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const SCIP_Real*      lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
    int                   lpnnonz,            /**< number of nonzero elements in the LP-constraint-matrix */
-   int*                  lprow,              /**< row-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   int*                  lpcol,              /**< column-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            lpval,              /**< values of LP-constraint-matrix entries, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            solvector,          /**< values of all variables (including fixed ones) in the solution that should be checked */
+   const int*            lpbeg,              /**< start index of each row in ind- and val-array, or NULL if nnonz == 0 */
+   const int*            lpind,              /**< column indices of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      lpval,              /**< values of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      solvector,          /**< values of all variables (including fixed ones) in the solution that should be checked */
    SCIP_Real             feastol,            /**< feasibility tolerance to check feasibility for */
    SCIP_Real             epsilon,            /**< tolerance used to check for fixed variables */
    SCIP_Bool*            infeasible          /**< pointer to store whether solution is feasible */
@@ -98,7 +99,6 @@ SCIP_RETCODE SCIPsdpSolcheckerCheck(
    int j;
    int b;
    int v;
-   int ind;
 
    assert( bufmem != NULL );
    assert( lb != NULL );
@@ -123,9 +123,9 @@ SCIP_RETCODE SCIPsdpSolcheckerCheck(
    assert( nlpcons >= 0 );
    assert( nlpcons == 0 || lplhs != NULL );
    assert( nlpcons == 0 || lprhs != NULL );
+   assert( nlpcons == 0 || lpbeg != NULL );
    assert( lpnnonz >= 0 );
-   assert( nlpcons == 0 || lprow != NULL );
-   assert( nlpcons == 0 || lpcol != NULL );
+   assert( nlpcons == 0 || lpind != NULL );
    assert( nlpcons == 0 || lpval != NULL );
    assert( solvector != NULL );
    assert( feastol >= 0 );
@@ -146,37 +146,34 @@ SCIP_RETCODE SCIPsdpSolcheckerCheck(
    /* check linear constraints (since DSDP sorts the lp-arrays by cols, we cannot expect them to be sorted) */
    if ( nlpcons > 0 )
    {
-      SCIP_Real* lpconsvals;
-
-      BMS_CALL( BMSallocBufferMemoryArray(bufmem, &lpconsvals, nlpcons) );
-
-      /* initialize all rows with zero */
-      for (i = 0; i < nlpcons; i++)
-         lpconsvals[i] = 0;
-
-      /* compute the values of all rows */
-      for (i = 0; i < lpnnonz; i++)
-      {
-         if ( lb[lpcol[i]] < ub[lpcol[i]] - epsilon ) /* fixed variables are already included in lhs/rhs */
-            lpconsvals[lprow[i]] += solvector[lpcol[i]] * lpval[i];
-      }
-
-      /* check all active constraints for feasibility */
-      ind = 0; /* used to iterate over active constraints */
       for (i = 0; i < nlpcons; i++)
       {
-         if ( lpconsvals[i] < lplhs[ind] - feastol || lpconsvals[i] > lprhs[ind] + feastol)
+         SCIP_Real lpconsvals = 0.0;
+         int nextbeg;
+
+         if ( lpindchanges[i] < 0 )
+            continue;
+
+         if ( i == nlpcons - 1 )
+            nextbeg = lpnnonz;
+         else
+            nextbeg = lpbeg[i + 1];
+
+         /* compute the values of all rows */
+         for (j = lpbeg[i]; j < nextbeg; j++)
+         {
+            if ( lb[lpind[j]] < ub[lpind[j]] - epsilon ) /* fixed variables are already included in lhs/rhs */
+               lpconsvals += solvector[lpind[j]] * lpval[j];
+         }
+
+         if ( lpconsvals < lplhs[i] - feastol || lpconsvals > lprhs[i] + feastol )
          {
             SCIPdebugMessage("solution found infeasible (feastol=%g) for dual lp constraint: LP-%d = %g <|= [%g,%g]\n",
-               feastol, i, lpconsvals[i], lplhs[ind], lprhs[ind]);
-            BMSfreeBufferMemoryArray(bufmem, &lpconsvals);
+               feastol, i, lpconsvals, lplhs[i], lprhs[i]);
             *infeasible = TRUE;
             return SCIP_OKAY;
          }
-
-         ind++;
       }
-      BMSfreeBufferMemoryArray(bufmem, &lpconsvals);
    }
 
    /* check sdp constraints */
@@ -277,37 +274,38 @@ SCIP_RETCODE SCIPsdpSolcheckerCheck(
 SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
    BMS_BUFMEM*           bufmem,             /**< buffer memory */
    int                   nvars,              /**< number of variables */
-   SCIP_Real*            lb,                 /**< lower bounds of variables */
-   SCIP_Real*            ub,                 /**< upper bounds of variables */
+   const SCIP_Real*      lb,                 /**< lower bounds of variables */
+   const SCIP_Real*      ub,                 /**< upper bounds of variables */
    int                   nsdpblocks,         /**< number of SDP-blocks */
-   int*                  sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
-   int*                  sdpnblockvars,      /**< number of variables that exist in each block */
+   const int*            sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
+   const int*            sdpnblockvars,      /**< number of variables that exist in each block */
    int                   sdpconstnnonz,      /**< number of nonzero elements in the constant matrices of the SDP-blocks AFTER FIXINGS */
-   int*                  sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
+   const int*            sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
                                               *   number of entries  of sdpconst row/col/val [i] AFTER FIXINGS */
-   int**                 sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
-   int**                 sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
-   SCIP_Real**           sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
+   int* const*           sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
+   int* const*           sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
+   SCIP_Real* const*     sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
    int                   sdpnnonz,           /**< number of nonzero elements in the SDP-constraint-matrix */
-   int**                 sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
+   int* const*           sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
                                               *   the number of entries of sdp row/col/val [i][j] */
-   int**                 sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
+   int* const*           sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
                                               *   in the i-th block */
-   int***                sdprow,             /**< pointer to the row-indices for each block and variable */
-   int***                sdpcol,             /**< pointer to the column-indices for each block and variable */
-   SCIP_Real***          sdpval,             /**< values of SDP-constraint matrix entries (may be NULL if sdpnnonz = 0) */
-   int**                 indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
+   int** const*          sdprow,             /**< pointer to the row-indices for each block and variable */
+   int** const*          sdpcol,             /**< pointer to the column-indices for each block and variable */
+   SCIP_Real** const*    sdpval,             /**< values of SDP-constraint matrix entries (may be NULL if sdpnnonz = 0) */
+   int* const*           indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
                                               *   be removed, otherwise it gives the number of indices removed before this */
-   int*                  nremovedinds,       /**< the number of rows/cols to be fixed for each block */
-   int*                  blockindchanges,    /**< block indizes will be modified by these, see indchanges */
+   const int*            nremovedinds,       /**< the number of rows/cols to be fixed for each block */
+   const int*            blockindchanges,    /**< block indizes will be modified by these, see indchanges */
    int                   nlpcons,            /**< number of active (at least two nonzeros) LP-constraints */
-   SCIP_Real*            lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
-   SCIP_Real*            lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const int*            lpindchanges,       /**< array for the number of LP-constraints removed before the current one (-1 if removed itself) */
+   const SCIP_Real*      lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const SCIP_Real*      lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
    int                   lpnnonz,            /**< number of nonzero elements in the LP-constraint-matrix */
-   int*                  lprow,              /**< row-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   int*                  lpcol,              /**< column-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            lpval,              /**< values of LP-constraint-matrix entries, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            solvector,          /**< values of all variables (including fixed ones) in the solution that should be checked */
+   const int*            lpbeg,              /**< start index of each row in ind- and val-array, or NULL if nnonz == 0 */
+   const int*            lpind,              /**< column indices of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      lpval,              /**< values of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      solvector,          /**< values of all variables (including fixed ones) in the solution that should be checked */
    SCIP_Real             feastol,            /**< feasibility tolerance to check feasibility for */
    SCIP_Real             epsilon,            /**< tolerance used to check for fixed variables */
    SCIP_Real*            maxabsviolbnds,     /**< pointer to store maximal absolute violation of variable bounds  */
@@ -323,7 +321,6 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
    int j;
    int b;
    int v;
-   int ind;
    SCIP_Real viol;
 
    assert( bufmem != NULL );
@@ -349,9 +346,9 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
    assert( nlpcons >= 0 );
    assert( nlpcons == 0 || lplhs != NULL );
    assert( nlpcons == 0 || lprhs != NULL );
+   assert( nlpcons == 0 || lpbeg != NULL );
    assert( lpnnonz >= 0 );
-   assert( nlpcons == 0 || lprow != NULL );
-   assert( nlpcons == 0 || lpcol != NULL );
+   assert( nlpcons == 0 || lpind != NULL );
    assert( nlpcons == 0 || lpval != NULL );
    assert( solvector != NULL );
    assert( feastol >= 0 );
@@ -386,77 +383,68 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
       if ( viol > *maxabsviolbnds )
          *maxabsviolbnds = viol;
 
-      if ( solvector[i] < lb[i] - feastol || solvector[i] > ub[i] + feastol )
+      if ( viol > feastol )
       {
          SCIPdebugMessage("solution found infeasible (feastol=%f) for dual variable bounds: x[%d] = %g <|= [%g, %g]\n",
             feastol, i, solvector[i], lb[i], ub[i]);
          *infeasible = TRUE;
+         return SCIP_OKAY;
       }
    }
 
-   /* check linear constraints (since DSDP sorts the lp-arrays by cols, we cannot expect them to be sorted) */
+   /* check linear constraints */
    if ( nlpcons > 0 )
    {
-      SCIP_Real* lpconsvals;
-
-      BMS_CALL( BMSallocBufferMemoryArray(bufmem, &lpconsvals, nlpcons) );
-
-      /* initialize all rows with zero */
-      for (i = 0; i < nlpcons; i++)
-         lpconsvals[i] = 0;
-
-      /* compute the values of all rows */
-      for (i = 0; i < lpnnonz; i++)
+      for (i = 0; i < nlpcons; ++i)
       {
-         if ( lb[lpcol[i]] < ub[lpcol[i]] - epsilon ) /* fixed variables are already included in lhs/rhs */
-            lpconsvals[lprow[i]] += solvector[lpcol[i]] * lpval[i];
-      }
+         SCIP_Real lpconsvals = 0.0;
+         int nextbeg;
 
-      /* check all active constraints for feasibility */
-      ind = 0; /* used to iterate over active constraints */
-      for (i = 0; i < nlpcons; i++)
-      {
-         viol = MAX3(lplhs[ind] - lpconsvals[i], lpconsvals[i] - lprhs[ind], 0.0);
+         if ( lpindchanges[i] < 0 )
+            continue;
+
+         if ( i == nlpcons - 1 )
+            nextbeg = lpnnonz;
+         else
+            nextbeg = lpbeg[i + 1];
+
+         /* compute the values of all rows */
+         for (j = lpbeg[i]; j < nextbeg; j++)
+         {
+            if ( lb[lpind[j]] < ub[lpind[j]] - epsilon ) /* fixed variables are already included in lhs/rhs */
+               lpconsvals += solvector[lpind[j]] * lpval[j];
+         }
+
+         viol = MAX3(lplhs[i] - lpconsvals, lpconsvals - lprhs[i], 0.0);
          *sumabsviolcons += viol;
 
-#ifdef SCIP_MORE_DEBUG
-         SCIPdebugMessage("Dual linear constraint %d: lhs = %g, rhs = %g, val = %.15g, viol = %.15g\n", i, lplhs[i], lprhs[i], lpconsvals[i], viol);
-#endif
-
          if ( viol > *maxabsviolcons )
             *maxabsviolcons = viol;
 
-         if ( lpconsvals[i] < lplhs[ind] - feastol || lpconsvals[i] > lprhs[ind] + feastol)
+         if ( viol > feastol )
          {
-            SCIPdebugMessage("solution found infeasible (feastol=%g) for dual lp constraint: LP-%d = %g <|= [%g,%g].\n",
-               feastol, i, lpconsvals[i], lplhs[ind], lprhs[ind]);
-            BMSfreeBufferMemoryArray(bufmem, &lpconsvals);
+            SCIPdebugMessage("solution found infeasible (feastol=%g) for dual lp constraint: LP-%d = %g <|= [%g,%g]\n",
+               feastol, i, lpconsvals, lplhs[i], lprhs[i]);
             *infeasible = TRUE;
+            return SCIP_OKAY;
          }
-
-         ind++;
       }
-      BMSfreeBufferMemoryArray(bufmem, &lpconsvals);
    }
 
-   /* check sdp constraints */
+   /* check SDP constraints */
    if ( nsdpblocks > 0 )
    {
       SCIP_Real* fullsdpmatrix;
       SCIP_Real eigenvalue;
       int maxblocksize = 0;
+      int row;
+      int col;
 
-      /* allocate memory */
-      if ( nsdpblocks == 1 )
-         maxblocksize = sdpblocksizes[0] - nremovedinds[0];
-      else
+      /* calculate maximum size of any SDP block to not have to reallocate memory in between */
+      for (b = 0; b < nsdpblocks; b++)
       {
-         /* calculate maximum size of any SDP block to not have to reallocate memory in between */
-         for (b = 0; b < nsdpblocks; b++)
-         {
-            if ( (sdpblocksizes[b] - nremovedinds[b]) > maxblocksize )
-               maxblocksize = sdpblocksizes[b] - nremovedinds[b];
-         }
+         if ( (sdpblocksizes[b] - nremovedinds[b]) > maxblocksize )
+            maxblocksize = sdpblocksizes[b] - nremovedinds[b];
       }
 
       BMS_CALL( BMSallocBufferMemoryArray(bufmem, &fullsdpmatrix, maxblocksize * maxblocksize) );/*lint !e647*/
@@ -465,13 +453,15 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
       {
          if ( blockindchanges[b] > -1 )
          {
+            int redsize;
+
+            redsize = sdpblocksizes[b] - nremovedinds[b];
+
             /* initialize lower triangular part of fullsdpmatrix with zero */
-            for (i = 0; i < sdpblocksizes[b] - nremovedinds[b]; i++)
+            for (i = 0; i < redsize; i++)
             {
                for (j = 0; j <= i; j++)
-               {
-                  fullsdpmatrix[i * (sdpblocksizes[b] - nremovedinds[b]) + j] = 0.0; /*lint !e679*/
-               }
+                  fullsdpmatrix[i * redsize + j] = 0.0; /*lint !e679*/
             }
 
             /* iterate over all non-fixed variables and add the corresponding nonzeros */
@@ -481,8 +471,17 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
                {
                   for (i = 0; i < sdpnblockvarnonz[b][v]; i++)
                   {
-                     fullsdpmatrix[((sdprow[b][v][i] - indchanges[b][sdprow[b][v][i]]) * (sdpblocksizes[b] - nremovedinds[b])) +
-                                   sdpcol[b][v][i] - indchanges[b][sdpcol[b][v][i]]] += solvector[sdpvar[b][v]] * sdpval[b][v][i];/*lint !e679*/
+                     row = sdprow[b][v][i];
+                     row = row - indchanges[b][row];
+
+                     col = sdpcol[b][v][i];
+                     col = col - indchanges[b][col];
+
+                     assert( 0 <= row && row < redsize );
+                     assert( 0 <= col && col < redsize );
+                     assert( row >= col );
+
+                     fullsdpmatrix[row * redsize + col] += solvector[sdpvar[b][v]] * sdpval[b][v][i];/*lint !e679*/
                   }
                }
             }
@@ -492,22 +491,29 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
             {
                for (i = 0; i < sdpconstnblocknonz[b]; i++)
                {
-                  fullsdpmatrix[((sdpconstrow[b][i] - indchanges[b][sdpconstrow[b][i]]) * (sdpblocksizes[b] - nremovedinds[b])) +
-                     sdpconstcol[b][i] - indchanges[b][sdpconstcol[b][i]]] -= sdpconstval[b][i];/*lint !e679*/
+                  row = sdpconstrow[b][i];
+                  row = row - indchanges[b][row];
+
+                  col = sdpconstcol[b][i];
+                  col = col - indchanges[b][col];
+
+                  assert( 0 <= row && row < redsize );
+                  assert( 0 <= col && col < redsize );
+                  assert( row >= col );
+
+                  fullsdpmatrix[row * redsize + col] -= sdpconstval[b][i];/*lint !e679*/
                }
             }
 
             /* extend to full symmetric matrix for LAPACK */
-            for (i = 0; i < sdpblocksizes[b] - nremovedinds[b]; i++)
+            for (i = 0; i < redsize; i++)
             {
                for (j = 0; j < i; j++)
-               {
-                  fullsdpmatrix[j * (sdpblocksizes[b] - nremovedinds[b]) + i] = fullsdpmatrix[i * (sdpblocksizes[b] - nremovedinds[b]) + j];/*lint !e679*/
-               }
+                  fullsdpmatrix[j * redsize + i] = fullsdpmatrix[i * redsize + j];/*lint !e679*/
             }
 
             /* compute smallest eigenvalue using LAPACK */
-            SCIP_CALL( SCIPlapackComputeIthEigenvalue(bufmem, FALSE, sdpblocksizes[b] - nremovedinds[b], fullsdpmatrix, 1, &eigenvalue, NULL) );
+            SCIP_CALL( SCIPlapackComputeIthEigenvalue(bufmem, FALSE, redsize, fullsdpmatrix, 1, &eigenvalue, NULL) );
 
             viol = MAX(-eigenvalue, 0.0);
             *sumabsviolsdp += viol;
@@ -519,12 +525,13 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
             if ( viol > *maxabsviolsdp )
                *maxabsviolsdp = viol;
 
-            if ( eigenvalue < - feastol )
+            if ( viol > feastol )
             {
                SCIPdebugMessage("solution found infeasible (feastol=%.10g) for dual SDP constraint %d, smallest eigenvector %.10g\n",
                      feastol, b, eigenvalue);
                BMSfreeBufferMemoryArray(bufmem, &fullsdpmatrix);
                *infeasible = TRUE;
+               return SCIP_OKAY;
             }
          }
       }
@@ -546,43 +553,44 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolDual(
 SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolPrimal(
    BMS_BUFMEM*           bufmem,             /**< buffer memory */
    int                   nvars,              /**< number of variables */
-   SCIP_Real*            obj,                /**< objective coefficients of variables (in dual problem) */
-   SCIP_Real*            lb,                 /**< lower bounds of variables */
-   SCIP_Real*            ub,                 /**< upper bounds of variables */
-   int*                  inputtomosekmapper, /**< entry i gives the index of input variable i in MOSEK (starting from 0) or
+   const SCIP_Real*      obj,                /**< objective coefficients of variables (in dual problem) */
+   const SCIP_Real*      lb,                 /**< lower bounds of variables */
+   const SCIP_Real*      ub,                 /**< upper bounds of variables */
+   const int*            inputtomosekmapper, /**< entry i gives the index of input variable i in MOSEK (starting from 0) or
                                               *   -j (j=1, 2, ..., nvars - nactivevars) if the variable is fixed, the value and objective value of
                                               *   this fixed variable can be found in entry j-1 of fixedval/obj */
    int                   nsdpblocks,         /**< number of SDP-blocks */
-   int*                  sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
-   int*                  sdpnblockvars,      /**< number of variables that exist in each block */
+   const int*            sdpblocksizes,      /**< sizes of the SDP-blocks (may be NULL if nsdpblocks = sdpconstnnonz = sdpnnonz = 0) */
+   const int*            sdpnblockvars,      /**< number of variables that exist in each block */
    int                   sdpconstnnonz,      /**< number of nonzero elements in the constant matrices of the SDP-blocks AFTER FIXINGS */
-   int*                  sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
+   const int*            sdpconstnblocknonz, /**< number of nonzeros for each variable in the constant part, also the i-th entry gives the
                                               *   number of entries  of sdpconst row/col/val [i] AFTER FIXINGS */
-   int**                 sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
-   int**                 sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
-   SCIP_Real**           sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
+   int* const*           sdpconstrow,        /**< pointers to row-indices for each block AFTER FIXINGS*/
+   int* const*           sdpconstcol,        /**< pointers to column-indices for each block AFTER FIXINGS */
+   SCIP_Real* const*     sdpconstval,        /**< pointers to the values of the nonzeros for each block AFTER FIXINGS */
    int                   sdpnnonz,           /**< number of nonzero elements in the SDP-constraint-matrix */
-   int**                 sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
-                                              *   the number of entries of sdp row/col/val [i][j] */
-   int**                 sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
-                                              *   in the i-th block */
-   int***                sdprow,             /**< pointer to the row-indices for each block and variable */
-   int***                sdpcol,             /**< pointer to the column-indices for each block and variable */
-   SCIP_Real***          sdpval,             /**< values of SDP-constraint matrix entries (may be NULL if sdpnnonz = 0) */
-   int**                 indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
-                                              *   be removed, otherwise it gives the number of indices removed before this */
-   int*                  nremovedinds,       /**< the number of rows/cols to be fixed for each block */
-   int*                  blockindchanges,    /**< block indizes will be modified by these, see indchanges */
+   int* const*           sdpnblockvarnonz,   /**< entry [i][j] gives the number of nonzeros for block i and variable j, this is exactly
+                                        *   the number of entries of sdp row/col/val [i][j] */
+   int* const*           sdpvar,             /**< sdpvar[i][j] gives the sdp-index of the j-th variable (according to the sorting for row/col/val)
+                                        *   in the i-th block */
+   int** const*          sdprow,             /**< pointer to the row-indices for each block and variable */
+   int** const*          sdpcol,             /**< pointer to the column-indices for each block and variable */
+   SCIP_Real** const*    sdpval,             /**< values of SDP-constraint matrix entries (may be NULL if sdpnnonz = 0) */
+   int* const*           indchanges,         /**< changes needed to be done to the indices, if indchanges[block][ind]=-1, then the index can
+                                        *   be removed, otherwise it gives the number of indices removed before this */
+   const int*            nremovedinds,       /**< the number of rows/cols to be fixed for each block */
+   const int*            blockindchanges,    /**< block indizes will be modified by these, see indchanges */
    int                   nremovedblocks,     /**< number of empty blocks that should be removed */
    int                   nlpcons,            /**< number of active (at least two nonzeros) LP-constraints */
-   SCIP_Real*            lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
-   SCIP_Real*            lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const int*            lpindchanges,       /**< array for the number of LP-constraints removed before the current one (-1 if removed itself) */
+   const SCIP_Real*      lplhs,              /**< left-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
+   const SCIP_Real*      lprhs,              /**< right-hand sides of active LP-rows after fixings (may be NULL if nlpcons = 0) */
    int                   lpnnonz,            /**< number of nonzero elements in the LP-constraint-matrix */
-   int*                  lprow,              /**< row-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   int*                  lpcol,              /**< column-index for each entry in lpval-array, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            lpval,              /**< values of LP-constraint-matrix entries, might get sorted (may be NULL if lpnnonz = 0) */
-   SCIP_Real*            solvector,          /**< values of all scalar variables in the solution that should be checked */
-   SCIP_Real**           solmatrices,        /**< values of all matrix variables in the solution that should be checked */
+   const int*            lpbeg,              /**< start index of each row in ind- and val-array, or NULL if nnonz == 0 */
+   const int*            lpind,              /**< column indices of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      lpval,              /**< values of constraint matrix entries, or NULL if nnonz == 0 */
+   const SCIP_Real*      solvector,          /**< values of all scalar variables in the solution that should be checked */
+   SCIP_Real* const*     solmatrices,        /**< values of all matrix variables in the solution that should be checked */
    SCIP_Real             feastol,            /**< feasibility tolerance to check feasibility for */
    SCIP_Real             epsilon,            /**< tolerance used to check for fixed variables */
    SCIP_Real*            maxabsviolbnds,     /**< pointer to store maximal absolute violation of variable bounds  */
@@ -616,7 +624,6 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolPrimal(
    int mosekcol;
    int blockvar;
    int k;
-   int nnonz;
    int c;
 
    assert( bufmem != NULL );
@@ -643,9 +650,9 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolPrimal(
    assert( nlpcons >= 0 );
    assert( nlpcons == 0 || lplhs != NULL );
    assert( nlpcons == 0 || lprhs != NULL );
+   assert( nlpcons == 0 || lpbeg != NULL );
    assert( lpnnonz >= 0 );
-   assert( nlpcons == 0 || lprow != NULL );
-   assert( nlpcons == 0 || lpcol != NULL );
+   assert( nlpcons == 0 || lpind != NULL );
    assert( nlpcons == 0 || lpval != NULL );
    assert( solvector != NULL );
    assert( feastol >= 0 );
@@ -838,44 +845,44 @@ SCIP_RETCODE SCIPsdpSolcheckerCheckAndGetViolPrimal(
       /* add the entries corresponding to the lp-constraints in the dual problem */
       if ( lpnnonz > 0 )
       {
-         int currentrow;
          int varcnt = 0;
-
-         currentrow = lprow[0];
-         for (nnonz = 0; nnonz < lpnnonz; ++nnonz)
+         for (i = 0; i < nlpcons; ++i)
          {
-            assert( nnonz == 0 || lprow[nnonz-1] <= lprow[nnonz] );  /* rows should be sorted */
-            assert( lprow[nnonz] == currentrow );
+            int nextbeg;
 
-            v = inputtomosekmapper[lpcol[nnonz]];
-            if ( v >= 0 )
+            if ( lpindchanges[i] < 0 )
+               continue;
+
+            if ( i == nlpcons - 1 )
+               nextbeg = lpnnonz;
+            else
+               nextbeg = lpbeg[i + 1];
+
+            for (j = lpbeg[i]; j < nextbeg; ++j)
             {
-               assert( v < nactivevars );
+               assert( 0 <= lpind[j] && lpind[j] < nvars );
+               v = inputtomosekmapper[lpind[j]];
+               if ( v >= 0 )
+               {
+                  assert( v < nactivevars );
 
-               /* treat left hand side */
-               if ( lplhs[lprow[nnonz]] > - INF )
-                  lpconsvals[v] += lpval[nnonz] * solvector[varcnt];
+                  /* treat left hand side */
+                  if ( lplhs[i] > - INF )
+                     lpconsvals[v] += lpval[j] * solvector[varcnt];
 
-               /* treat right hand side */
-               if ( lprhs[lprow[nnonz]] < INF )
-                  lpconsvals[v] -= lpval[nnonz] * solvector[varcnt + 1];
+                  /* treat right hand side */
+                  if ( lprhs[i] < INF )
+                     lpconsvals[v] -= lpval[j] * solvector[varcnt + 1];
+               }
             }
 
-            /* we finished a row */
-            if ( nnonz == lpnnonz - 1 || lprow[nnonz + 1] > currentrow )
-            {
-               /* treat left hand side */
-               if ( lplhs[currentrow] > - INF )
-                  varcnt++;
+            /* treat left hand side */
+            if ( lplhs[i] > - INF )
+               varcnt++;
 
-               /* treat right hand side */
-               if ( lprhs[currentrow] < INF )
-                  varcnt++;
-
-               /* reset counters */
-               if ( nnonz < lpnnonz )
-                  currentrow = lprow[nnonz+1];
-            }
+            /* treat right hand side */
+            if ( lprhs[i] < INF )
+               varcnt++;
          }
          assert( varcnt == nlpvars );
       }
